package at.jku.ssw.map;

import at.jku.ssw.Iterator;

/**
 * An object that associates String keys with Object values. A map cannot
 * contain duplicate keys; each key can map to at most one value.
 */
public abstract class Map {

	/**
	 * Returns true if this map contains an association for the specified key.
	 *
	 * @throws NullPointerException
	 *             if the provided key is null.
	 */
	public abstract boolean containsKey(Object key);

	/**
	 * Retrieves the value associated with the specified key.
	 *
	 * Returns null if the map contains no mapping for this key. A return value
	 * of null does not necessarily indicate that the map contains no mapping
	 * for the key; it's also possible that the map explicitly maps the key to
	 * null. The containsKey operation can be used to distinguish these two
	 * cases.
	 *
	 * @throws NullPointerException
	 *             if the provided key is null.
	 */
	public abstract Object get(Object key);

	/**
	 * Creates an association of the specified key to the specified value in
	 * this map. If an association for the specified key is already present, it
	 * is replaced.
	 *
	 * @throws NullPointerException
	 *             if the provided key is null.
	 * @throws IllegalStateException
	 *             if the map is full.
	 */
	public abstract void put(Object key, Object value);

	/**
	 * If this map contains an association for the specified key, removes it and
	 * returns the formerly associated value.
	 *
	 * @throws NullPointerException
	 *             if the provided key is null.
	 */
	public abstract Object remove(Object key);

	/**
	 * Returns the number of elements stored in this map.
	 * 
	 * @return the number of elements in this hashmap.
	 */
	public abstract int size();

	/**
	 * Returns an iterator of <code>Pair<Key,Value> pairs</code> stored in this
	 * map.
	 * 
	 * @return {@linkplain Iterator} of {@linkplain Pair} elements in this map.
	 */
	public abstract Iterator iterator();

	/**
	 * Returns a string representation of this map.
	 */
	@Override
	public String toString() {
		if (size() <= 0) {
			return "{}";
		}
		StringBuilder sb = new StringBuilder();
		sb.append("{");
		Iterator it = iterator();
		Pair pair = (Pair) it.next();
		sb.append(pair.key).append('=').append(pair.value);
		while (it.hasNext()) {
			Pair entry = (Pair) it.next();
			sb.append(", ").append(entry.key).append(": ").append(entry.value);
		}
		sb.append("}");
		return sb.toString();
	}
}
